@@ -3,7 +3,7 @@ use markup5ever::{expanded_name, local_name, namespace_url, ns};
 
 use crate::document::AcquiredNodes;
 use crate::drawing_ctx::DrawingCtx;
-use crate::element::{set_attribute, SetAttributes};
+use crate::element::{set_attribute, ElementTrait};
 use crate::error::*;
 use crate::node::{CascadedValues, Node};
 use crate::parsers::{Parse, ParseValue};
@@ -49,7 +49,7 @@ pub struct DisplacementMap {
     color_interpolation_filters: ColorInterpolationFilters,
 }
 
-impl SetAttributes for FeDisplacementMap {
+impl ElementTrait for FeDisplacementMap {
     fn set_attributes(&mut self, attrs: &Attributes, session: &Session) {
         let (in1, in2) = self.base.parse_two_inputs(attrs, session);
         self.params.in1 = in1;
@@ -124,7 +124,7 @@ impl DisplacementMap {
             input_1.surface().surface_type(),
         )?;
 
-        surface.draw::<cairo::Error>(&mut |cr| {
+        surface.draw(&mut |cr| {
             for (x, y, displacement_pixel) in Pixels::within(&displacement_surface, bounds) {
                 let get_value = |channel| match channel {
                     ColorChannel::R => displacement_pixel.r,
