@@ -10,7 +10,7 @@ use std::sync::Arc;
 use crate::bbox::BoundingBox;
 use crate::document::{AcquiredNodes, NodeId};
 use crate::drawing_ctx::{create_pango_context, DrawingCtx, FontOptions, ViewParams};
-use crate::element::{set_attribute, Draw, Element, SetAttributes};
+use crate::element::{set_attribute, ElementData, ElementTrait};
 use crate::error::*;
 use crate::layout::{self, FontProperties, StackingContext, Stroke, TextSpan};
 use crate::length::*;
@@ -24,7 +24,7 @@ use crate::properties::{
 use crate::rect::Rect;
 use crate::session::Session;
 use crate::space::{xml_space_normalize, NormalizeDefault, XmlSpaceNormalize};
-use crate::transform::Transform;
+use crate::transform::{Transform, ValidTransform};
 use crate::xml::Attributes;
 
 /// The state of a text layout operation.
@@ -33,7 +33,7 @@ struct LayoutContext {
     writing_mode: WritingMode,
 
     /// Current transform in the DrawingCtx.
-    transform: Transform,
+    transform: ValidTransform,
 
     /// Font options from the DrawingCtx.
     font_options: FontOptions,
@@ -144,7 +144,7 @@ impl MeasuredChunk {
         let mut measured_spans: Vec<MeasuredSpan> = chunk
             .spans
             .iter()
-            .map(|span| MeasuredSpan::from_span(layout_context, span))
+            .filter_map(|span| MeasuredSpan::from_span(layout_context, span))
             .collect();
 
         // The first span contains the (dx, dy) that will be applied to the whole chunk.
@@ -340,8 +340,23 @@ impl Span {
     }
 }
 
+/// Use as `PangoUnits::from_pixels()` so that we can check for overflow.
+struct PangoUnits(i32);
+
+impl PangoUnits {
+    fn from_pixels(v: f64) -> Option<Self> {
+        // We want (v * f64::from(pango::SCALE) + 0.5) as i32
+        //
+        // But check for overflow.
+
+        cast::i32(v * f64::from(pango::SCALE) + 0.5)
+            .ok()
+            .map(PangoUnits)
+    }
+}
+
 impl MeasuredSpan {
-    fn from_span(layout_context: &LayoutContext, span: &Span) -> MeasuredSpan {
+    fn from_span(layout_context: &LayoutContext, span: &Span) -> Option<MeasuredSpan> {
         let values = span.values.clone();
 
         let params = NormalizeParams::new(&values, &layout_context.view_params);
@@ -354,30 +369,31 @@ impl MeasuredSpan {
         );
 
         let with_control_chars = wrap_with_direction_control_chars(&span.text, &bidi_control);
-        let layout = create_pango_layout(layout_context, &properties, &with_control_chars);
-        let (w, h) = layout.size();
 
-        let w = f64::from(w) / f64::from(pango::SCALE);
-        let h = f64::from(h) / f64::from(pango::SCALE);
+        if let Some(layout) = create_pango_layout(layout_context, &properties, &with_control_chars)
+        {
+            let (w, h) = layout.size();
 
-        // This is the logical size of the layout, regardless of text direction, so it's always positive.
-        assert!(w >= 0.0);
-        assert!(h >= 0.0);
+            let w = f64::from(w) / f64::from(pango::SCALE);
+            let h = f64::from(h) / f64::from(pango::SCALE);
 
-        let advance = if layout_context.writing_mode.is_horizontal() {
-            (w, 0.0)
+            let advance = if layout_context.writing_mode.is_horizontal() {
+                (w, 0.0)
+            } else {
+                (0.0, w)
+            };
+
+            Some(MeasuredSpan {
+                values,
+                layout,
+                layout_size: (w, h),
+                advance,
+                dx: span.dx,
+                dy: span.dy,
+                link_target: span.link_target.clone(),
+            })
         } else {
-            (0.0, w)
-        };
-
-        MeasuredSpan {
-            values,
-            layout,
-            layout_size: (w, h),
-            advance,
-            dx: span.dx,
-            dy: span.dy,
-            link_target: span.link_target.clone(),
+            None
         }
     }
 }
@@ -448,7 +464,7 @@ impl PositionedSpan {
 
         let gravity = layout.context().gravity();
 
-        let bbox = compute_text_box(&layout, x, y, layout_context.transform, gravity);
+        let bbox = compute_text_box(&layout, x, y, *layout_context.transform, gravity);
 
         let stroke_paint = self.values.stroke().0.resolve(
             acquired_nodes,
@@ -521,8 +537,8 @@ fn children_to_chunks(
         } else {
             assert!(child.is_element());
 
-            match *child.borrow_element() {
-                Element::TSpan(ref tspan) => {
+            match *child.borrow_element_data() {
+                ElementData::TSpan(ref tspan) => {
                     let cascaded = CascadedValues::clone_with_node(cascaded, &child);
                     tspan.to_chunks(
                         &child,
@@ -537,7 +553,7 @@ fn children_to_chunks(
                     );
                 }
 
-                Element::Link(ref link) => {
+                ElementData::Link(ref link) => {
                     // TSpan::default sets all offsets to 0,
                     // which is what we want in links.
                     //
@@ -561,7 +577,7 @@ fn children_to_chunks(
                     );
                 }
 
-                Element::TRef(ref tref) => {
+                ElementData::TRef(ref tref) => {
                     let cascaded = CascadedValues::clone_with_node(cascaded, &child);
                     tref.to_chunks(
                         &child,
@@ -732,7 +748,7 @@ impl Text {
     }
 }
 
-impl SetAttributes for Text {
+impl ElementTrait for Text {
     fn set_attributes(&mut self, attrs: &Attributes, session: &Session) {
         for (attr, value) in attrs.iter() {
             match attr.expanded() {
@@ -744,9 +760,7 @@ impl SetAttributes for Text {
             }
         }
     }
-}
 
-impl Draw for Text {
     fn draw(
         &self,
         node: &Node,
@@ -815,7 +829,7 @@ impl Draw for Text {
                     }
                 }
 
-                let empty_bbox = BoundingBox::new().with_transform(*transform);
+                let empty_bbox = BoundingBox::new().with_transform(**transform);
 
                 let text_bbox = layout_spans.iter().fold(empty_bbox, |mut bbox, span| {
                     if let Some(ref span_bbox) = span.bbox {
@@ -923,7 +937,7 @@ fn extract_chars_children_to_chunks_recursively(
     }
 }
 
-impl SetAttributes for TRef {
+impl ElementTrait for TRef {
     fn set_attributes(&mut self, attrs: &Attributes, _session: &Session) {
         self.link = attrs
             .iter()
@@ -935,8 +949,6 @@ impl SetAttributes for TRef {
     }
 }
 
-impl Draw for TRef {}
-
 #[derive(Default)]
 pub struct TSpan {
     x: Option<Length<Horizontal>>,
@@ -989,7 +1001,7 @@ impl TSpan {
     }
 }
 
-impl SetAttributes for TSpan {
+impl ElementTrait for TSpan {
     fn set_attributes(&mut self, attrs: &Attributes, session: &Session) {
         for (attr, value) in attrs.iter() {
             match attr.expanded() {
@@ -1003,12 +1015,6 @@ impl SetAttributes for TSpan {
     }
 }
 
-impl Draw for TSpan {}
-
-fn to_pango_units(v: f64) -> i32 {
-    (v * f64::from(pango::SCALE) + 0.5) as i32
-}
-
 impl From<FontStyle> for pango::Style {
     fn from(s: FontStyle) -> pango::Style {
         match s {
@@ -1191,11 +1197,12 @@ fn wrap_with_direction_control_chars(s: &str, bidi_control: &BidiControl) -> Str
     res
 }
 
+/// Returns `None` if the layout would be invalid due to, for example, out-of-bounds font sizes.
 fn create_pango_layout(
     layout_context: &LayoutContext,
     props: &FontProperties,
     text: &str,
-) -> pango::Layout {
+) -> Option<pango::Layout> {
     let pango_context =
         create_pango_context(&layout_context.font_options, &layout_context.transform);
 
@@ -1221,14 +1228,37 @@ fn create_pango_layout(
 
     let layout = pango::Layout::new(&pango_context);
 
-    let attr_list = pango::AttrList::new();
-    add_pango_attributes(&attr_list, props, 0, text.len());
+    let font_size = PangoUnits::from_pixels(props.font_size);
+    let letter_spacing = PangoUnits::from_pixels(props.letter_spacing);
 
-    layout.set_attributes(Some(&attr_list));
-    layout.set_text(text);
-    layout.set_auto_dir(false);
+    if font_size.is_none() {
+        rsvg_log!(
+            &layout_context.session,
+            "font-size {} is out of bounds; ignoring span",
+            props.font_size
+        );
+    }
 
-    layout
+    if letter_spacing.is_none() {
+        rsvg_log!(
+            &layout_context.session,
+            "letter-spacing {} is out of bounds; ignoring span",
+            props.letter_spacing
+        );
+    }
+
+    if let (Some(font_size), Some(letter_spacing)) = (font_size, letter_spacing) {
+        let attr_list = pango::AttrList::new();
+        add_pango_attributes(&attr_list, props, 0, text.len(), font_size, letter_spacing);
+
+        layout.set_attributes(Some(&attr_list));
+        layout.set_text(text);
+        layout.set_auto_dir(false);
+
+        Some(layout)
+    } else {
+        None
+    }
 }
 
 /// Adds Pango attributes, suitable for a span of text, to an `AttrList`.
@@ -1237,6 +1267,8 @@ fn add_pango_attributes(
     props: &FontProperties,
     start_index: usize,
     end_index: usize,
+    font_size: PangoUnits,
+    letter_spacing: PangoUnits,
 ) {
     let start_index = u32::try_from(start_index).expect("Pango attribute index must fit in u32");
     let end_index = u32::try_from(end_index).expect("Pango attribute index must fit in u32");
@@ -1255,12 +1287,11 @@ fn add_pango_attributes(
     font_desc.set_weight(pango::Weight::from(props.font_weight));
     font_desc.set_stretch(pango::Stretch::from(props.font_stretch));
 
-    font_desc.set_size(to_pango_units(props.font_size));
+    font_desc.set_size(font_size.0);
 
     attributes.push(pango::AttrFontDesc::new(&font_desc).upcast());
 
-    attributes
-        .push(pango::AttrInt::new_letter_spacing(to_pango_units(props.letter_spacing)).upcast());
+    attributes.push(pango::AttrInt::new_letter_spacing(letter_spacing.0).upcast());
 
     if props.text_decoration.overline {
         attributes.push(pango::AttrInt::new_overline(pango::Overline::Single).upcast());
@@ -1410,4 +1441,14 @@ mod tests {
             (0.0, -4.0)
         );
     }
+
+    #[test]
+    fn pango_units_works() {
+        assert_eq!(PangoUnits::from_pixels(10.0).unwrap().0, pango::SCALE * 10);
+    }
+
+    #[test]
+    fn pango_units_detects_overflow() {
+        assert!(PangoUnits::from_pixels(1e7).is_none());
+    }
 }
