@@ -5,7 +5,7 @@ use std::fmt;
 use std::io::BufReader;
 use std::path::{Path, PathBuf};
 
-use librsvg::surface_utils::shared_surface::{SharedImageSurface, SurfaceType};
+use rsvg::surface_utils::shared_surface::{SharedImageSurface, SurfaceType};
 
 use crate::compare_surfaces::BufferDiff;
 use crate::reference_utils::{surface_from_png, Compare, Deviation, Reference};
